/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

package backends

import (
	"net/url"
	"strings"

	"github.com/spf13/viper"
	"go.uploadedlobster.com/scotty/models"
	"golang.org/x/oauth2"
)

func BuildRedirectURL(config *viper.Viper, backend string) (*url.URL, error) {
	callbackHost, _ := strings.CutSuffix(config.GetString("oauth-host"), "/")
	if callbackHost == "" {
		callbackHost = "127.0.0.1:2369"
	}
	callbackPath := "/callback/" + backend
	return url.Parse("http://" + callbackHost + callbackPath)
}

func Authenticate(backend models.Backend, token *oauth2.Token, config *viper.Viper) (bool, error) {
	authenticator, auth := backend.(models.OAuth2Authenticator)
	if auth {
		// FIXME
		redirectURL, err := BuildRedirectURL(config, backend.Name())
		if err != nil {
			return auth, err
		}
		authenticator.OAuth2Setup(redirectURL.String(), token)
	}
	return auth, nil
}
